@@ -2,9 +2,11 @@
 #include <inttypes.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include "nvme-print.h"
 #include "json.h"
+#include "nvme-models.h"
 
 static long double int128_to_double(__u8 *data)
 {
@@ -59,6 +61,20 @@ void d_raw(unsigned char *buf, unsigned len)
 		putchar(*(buf+i));
 }
 
+static void format(char *formatter, size_t fmt_sz, char *tofmt, size_t tofmtsz)
+{
+
+	snprintf(formatter,fmt_sz, "%-*.*s",
+		 (int)tofmtsz, (int)tofmtsz, tofmt);
+	/* trim() the obnoxious trailing white lines */
+	while (--fmt_sz) {
+		if (formatter[fmt_sz - 1] != ' ' && formatter[fmt_sz - 1] != '\0') {
+			formatter[fmt_sz] = '\0';
+			break;
+		}
+	}
+}
+
 static void show_nvme_id_ctrl_cmic(__u8 cmic)
 {
 	__u8 rsvd = (cmic & 0xF8) >> 3;
@@ -76,7 +92,7 @@ static void show_nvme_id_ctrl_cmic(__u8 cmic)
 
 static void show_nvme_id_ctrl_oaes(__le32 ctrl_oaes)
 {
-	__u32 oaes = le32toh(ctrl_oaes);
+	__u32 oaes = le32_to_cpu(ctrl_oaes);
 	__u32 rsvd0 = (oaes & 0xFFFFFE00) >> 9;
 	__u32 nace = (oaes & 0x100) >> 8;
 	__u32 rsvd1 = oaes & 0xFF;
@@ -92,7 +108,7 @@ static void show_nvme_id_ctrl_oaes(__le32 ctrl_oaes)
 
 static void show_nvme_id_ctrl_oacs(__le16 ctrl_oacs)
 {
-	__u16 oacs = le16toh(ctrl_oacs);
+	__u16 oacs = le16_to_cpu(ctrl_oacs);
 	__u16 rsvd = (oacs & 0xFFF0) >> 4;
 	__u16 nsm = (oacs & 0x8) >> 3;
 	__u16 fwc = (oacs & 0x4) >> 2;
@@ -166,7 +182,7 @@ static void show_nvme_id_ctrl_apsta(__u8 apsta)
 
 static void show_nvme_id_ctrl_rpmbs(__le32 ctrl_rpmbs)
 {
-	__u32 rpmbs = le32toh(ctrl_rpmbs);
+	__u32 rpmbs = le32_to_cpu(ctrl_rpmbs);
 	__u32 asz = (rpmbs & 0xFF000000) >> 24;
 	__u32 tsz = (rpmbs & 0xFF0000) >> 16;
 	__u32 rsvd = (rpmbs & 0xFFC0) >> 6;
@@ -202,7 +218,7 @@ static void show_nvme_id_ctrl_cqes(__u8 cqes)
 
 static void show_nvme_id_ctrl_oncs(__le16 ctrl_oncs)
 {
-	__u16 oncs = le16toh(ctrl_oncs);
+	__u16 oncs = le16_to_cpu(ctrl_oncs);
 	__u16 rsvd = (oncs & 0xFFC0) >> 6;
 	__u16 resv = (oncs & 0x20) >> 5;
 	__u16 save = (oncs & 0x10) >> 4;
@@ -230,7 +246,7 @@ static void show_nvme_id_ctrl_oncs(__le16 ctrl_oncs)
 
 static void show_nvme_id_ctrl_fuses(__le16 ctrl_fuses)
 {
-	__u16 fuses = le16toh(ctrl_fuses);
+	__u16 fuses = le16_to_cpu(ctrl_fuses);
 	__u16 rsvd = (fuses & 0xFE) >> 1;
 	__u16 cmpw = fuses & 0x1;
 
@@ -282,7 +298,7 @@ static void show_nvme_id_ctrl_nvscc(__u8 nvscc)
 
 static void show_nvme_id_ctrl_sgls(__le32 ctrl_sgls)
 {
-	__u32 sgls = le32toh(ctrl_sgls);
+	__u32 sgls = le32_to_cpu(ctrl_sgls);
 	__u32 rsvd0 = (sgls & 0xFFF80000) >> 19;
 	__u32 sglltb = (sgls & 0x40000) >> 18;
 	__u32 bacmdb = (sgls & 0x20000) >> 17;
@@ -450,9 +466,9 @@ void show_nvme_id_ns(struct nvme_id_ns *ns, unsigned int mode)
 	int human = mode&HUMAN,
 		vs = mode&VS;
 
-	printf("nsze    : %#"PRIx64"\n", (uint64_t)le64toh(ns->nsze));
-	printf("ncap    : %#"PRIx64"\n", (uint64_t)le64toh(ns->ncap));
-	printf("nuse    : %#"PRIx64"\n", (uint64_t)le64toh(ns->nuse));
+	printf("nsze    : %#"PRIx64"\n", (uint64_t)le64_to_cpu(ns->nsze));
+	printf("ncap    : %#"PRIx64"\n", (uint64_t)le64_to_cpu(ns->ncap));
+	printf("nuse    : %#"PRIx64"\n", (uint64_t)le64_to_cpu(ns->nuse));
 	printf("nsfeat  : %#x\n", ns->nsfeat);
 	if (human)
 		show_nvme_id_ns_nsfeat(ns->nsfeat);
@@ -478,12 +494,12 @@ void show_nvme_id_ns(struct nvme_id_ns *ns, unsigned int mode)
 	printf("fpi     : %#x\n", ns->fpi);
 	if (human)
 		show_nvme_id_ns_fpi(ns->fpi);
-	printf("nawun   : %d\n", le16toh(ns->nawun));
-	printf("nawupf  : %d\n", le16toh(ns->nawupf));
-	printf("nacwu   : %d\n", le16toh(ns->nacwu));
-	printf("nabsn   : %d\n", le16toh(ns->nabsn));
-	printf("nabo    : %d\n", le16toh(ns->nabo));
-	printf("nabspf  : %d\n", le16toh(ns->nabspf));
+	printf("nawun   : %d\n", le16_to_cpu(ns->nawun));
+	printf("nawupf  : %d\n", le16_to_cpu(ns->nawupf));
+	printf("nacwu   : %d\n", le16_to_cpu(ns->nacwu));
+	printf("nabsn   : %d\n", le16_to_cpu(ns->nabsn));
+	printf("nabo    : %d\n", le16_to_cpu(ns->nabo));
+	printf("nabspf  : %d\n", le16_to_cpu(ns->nabspf));
 	printf("nvmcap  : %.0Lf\n", int128_to_double(ns->nvmcap));
 
 	printf("nguid   : ");
@@ -500,14 +516,14 @@ void show_nvme_id_ns(struct nvme_id_ns *ns, unsigned int mode)
 		if (human)
 			printf("LBA Format %2d : Metadata Size: %-3d bytes - "
 				"Data Size: %-2d bytes - Relative Performance: %#x %s %s\n", i,
-				le16toh(ns->lbaf[i].ms), 1 << ns->lbaf[i].ds, ns->lbaf[i].rp,
+				le16_to_cpu(ns->lbaf[i].ms), 1 << ns->lbaf[i].ds, ns->lbaf[i].rp,
 				ns->lbaf[i].rp == 3 ? "Degraded" :
 				ns->lbaf[i].rp == 2 ? "Good" :
 				ns->lbaf[i].rp == 1 ? "Better" : "Best",
 				i == (ns->flbas & 0xf) ? "(in use)" : "");
 		else
 			printf("lbaf %2d : ms:%-3d lbads:%-2d rp:%#x %s\n", i,
-				le16toh(ns->lbaf[i].ms), ns->lbaf[i].ds, ns->lbaf[i].rp,
+				le16_to_cpu(ns->lbaf[i].ms), ns->lbaf[i].ds, ns->lbaf[i].rp,
 				i == (ns->flbas & 0xf) ? "(in use)" : "");
 	}
 	if (vs) {
@@ -518,7 +534,7 @@ void show_nvme_id_ns(struct nvme_id_ns *ns, unsigned int mode)
 
 static void print_ps_power_and_scale(__le16 ctr_power, __u8 scale)
 {
-	__u16 power = le16toh(ctr_power);
+	__u16 power = le16_to_cpu(ctr_power);
 
 	switch (scale & 0x3) {
 	case 0:
@@ -547,7 +563,7 @@ static void show_nvme_id_ctrl_power(struct nvme_id_ctrl *ctrl)
 
 
 	for (i = 0; i <= ctrl->npss; i++) {
-		__u16 max_power = le16toh(ctrl->psd[i].max_power);
+		__u16 max_power = le16_to_cpu(ctrl->psd[i].max_power);
 
 		printf("ps %4d : mp:", i);
 
@@ -561,7 +577,7 @@ static void show_nvme_id_ctrl_power(struct nvme_id_ctrl *ctrl)
 
 		printf("operational enlat:%d exlat:%d rrt:%d rrl:%d\n"
 			"          rwt:%d rwl:%d idle_power:",
-			le32toh(ctrl->psd[i].entry_lat), le32toh(ctrl->psd[i].exit_lat),
+			le32_to_cpu(ctrl->psd[i].entry_lat), le32_to_cpu(ctrl->psd[i].exit_lat),
 			ctrl->psd[i].read_tput, ctrl->psd[i].read_lat,
 			ctrl->psd[i].write_tput, ctrl->psd[i].write_lat);
 		print_ps_power_and_scale(ctrl->psd[i].idle_power,
@@ -578,8 +594,8 @@ void __show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode, void (*ve
 {
 	int human = mode & HUMAN, vs = mode & VS;
 
-	printf("vid     : %#x\n", le16toh(ctrl->vid));
-	printf("ssvid   : %#x\n", le16toh(ctrl->ssvid));
+	printf("vid     : %#x\n", le16_to_cpu(ctrl->vid));
+	printf("ssvid   : %#x\n", le16_to_cpu(ctrl->ssvid));
 	printf("sn      : %-.*s\n", (int)sizeof(ctrl->sn), ctrl->sn);
 	printf("mn      : %-.*s\n", (int)sizeof(ctrl->mn), ctrl->mn);
 	printf("fr      : %-.*s\n", (int)sizeof(ctrl->fr), ctrl->fr);
@@ -590,14 +606,14 @@ void __show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode, void (*ve
 	if (human)
 		show_nvme_id_ctrl_cmic(ctrl->cmic);
 	printf("mdts    : %d\n", ctrl->mdts);
-	printf("cntlid  : %x\n", le16toh(ctrl->cntlid));
-	printf("ver     : %x\n", le32toh(ctrl->ver));
-	printf("rtd3r   : %x\n", le32toh(ctrl->rtd3r));
-	printf("rtd3e   : %x\n", le32toh(ctrl->rtd3e));
-	printf("oaes    : %#x\n", le32toh(ctrl->oaes));
+	printf("cntlid  : %x\n", le16_to_cpu(ctrl->cntlid));
+	printf("ver     : %x\n", le32_to_cpu(ctrl->ver));
+	printf("rtd3r   : %x\n", le32_to_cpu(ctrl->rtd3r));
+	printf("rtd3e   : %x\n", le32_to_cpu(ctrl->rtd3e));
+	printf("oaes    : %#x\n", le32_to_cpu(ctrl->oaes));
 	if (human)
 		show_nvme_id_ctrl_oaes(ctrl->oaes);
-	printf("oacs    : %#x\n", le16toh(ctrl->oacs));
+	printf("oacs    : %#x\n", le16_to_cpu(ctrl->oacs));
 	if (human)
 		show_nvme_id_ctrl_oacs(ctrl->oacs);
 	printf("acl     : %d\n", ctrl->acl);
@@ -616,14 +632,14 @@ void __show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode, void (*ve
 	printf("apsta   : %#x\n", ctrl->apsta);
 	if (human)
 		show_nvme_id_ctrl_apsta(ctrl->apsta);
-	printf("wctemp  : %d\n", le16toh(ctrl->wctemp));
-	printf("cctemp  : %d\n", le16toh(ctrl->cctemp));
-	printf("mtfa    : %d\n", le16toh(ctrl->mtfa));
-	printf("hmpre   : %d\n", le32toh(ctrl->hmpre));
-	printf("hmmin   : %d\n", le32toh(ctrl->hmmin));
+	printf("wctemp  : %d\n", le16_to_cpu(ctrl->wctemp));
+	printf("cctemp  : %d\n", le16_to_cpu(ctrl->cctemp));
+	printf("mtfa    : %d\n", le16_to_cpu(ctrl->mtfa));
+	printf("hmpre   : %d\n", le32_to_cpu(ctrl->hmpre));
+	printf("hmmin   : %d\n", le32_to_cpu(ctrl->hmmin));
 	printf("tnvmcap : %.0Lf\n", int128_to_double(ctrl->tnvmcap));
 	printf("unvmcap : %.0Lf\n", int128_to_double(ctrl->unvmcap));
-	printf("rpmbs   : %#x\n", le32toh(ctrl->rpmbs));
+	printf("rpmbs   : %#x\n", le32_to_cpu(ctrl->rpmbs));
 	if (human)
 		show_nvme_id_ctrl_rpmbs(ctrl->rpmbs);
 	printf("sqes    : %#x\n", ctrl->sqes);
@@ -632,11 +648,11 @@ void __show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode, void (*ve
 	printf("cqes    : %#x\n", ctrl->cqes);
 	if (human)
 		show_nvme_id_ctrl_cqes(ctrl->cqes);
-	printf("nn      : %d\n", le32toh(ctrl->nn));
-	printf("oncs    : %#x\n", le16toh(ctrl->oncs));
+	printf("nn      : %d\n", le32_to_cpu(ctrl->nn));
+	printf("oncs    : %#x\n", le16_to_cpu(ctrl->oncs));
 	if (human)
 		show_nvme_id_ctrl_oncs(ctrl->oncs);
-	printf("fuses   : %#x\n", le16toh(ctrl->fuses));
+	printf("fuses   : %#x\n", le16_to_cpu(ctrl->fuses));
 	if (human)
 		show_nvme_id_ctrl_fuses(ctrl->fuses);
 	printf("fna     : %#x\n", ctrl->fna);
@@ -645,16 +661,18 @@ void __show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode, void (*ve
 	printf("vwc     : %#x\n", ctrl->vwc);
 	if (human)
 		show_nvme_id_ctrl_vwc(ctrl->vwc);
-	printf("awun    : %d\n", le16toh(ctrl->awun));
-	printf("awupf   : %d\n", le16toh(ctrl->awupf));
+	printf("awun    : %d\n", le16_to_cpu(ctrl->awun));
+	printf("awupf   : %d\n", le16_to_cpu(ctrl->awupf));
 	printf("nvscc   : %d\n", ctrl->nvscc);
 	if (human)
 		show_nvme_id_ctrl_nvscc(ctrl->nvscc);
-	printf("acwu    : %d\n", le16toh(ctrl->acwu));
-	printf("sgls    : %x\n", le32toh(ctrl->sgls));
+	printf("acwu    : %d\n", le16_to_cpu(ctrl->acwu));
+	printf("sgls    : %x\n", le32_to_cpu(ctrl->sgls));
 	if (human)
 		show_nvme_id_ctrl_sgls(ctrl->sgls);
 
+	printf("subnqn  : %-.*s\n", (int)sizeof(ctrl->subnqn), ctrl->subnqn);
+
 	show_nvme_id_ctrl_power(ctrl);
 	if (vendor_show)
 		vendor_show(ctrl->vs);
@@ -679,13 +697,13 @@ void show_error_log(struct nvme_error_log_page *err_log, int entries, const char
 	for (i = 0; i < entries; i++) {
 		printf(" Entry[%2d]   \n", i);
 		printf(".................\n");
-		printf("error_count  : %"PRIu64"\n", (uint64_t)le64toh(err_log[i].error_count));
+		printf("error_count  : %"PRIu64"\n", (uint64_t)le64_to_cpu(err_log[i].error_count));
 		printf("sqid         : %d\n", err_log[i].sqid);
 		printf("cmdid        : %#x\n", err_log[i].cmdid);
 		printf("status_field : %#x(%s)\n", err_log[i].status_field,
 			nvme_status_to_string(err_log[i].status_field >> 1));
 		printf("parm_err_loc : %#x\n", err_log[i].parm_error_location);
-		printf("lba          : %#"PRIx64"\n",(uint64_t)le64toh(err_log[i].lba));
+		printf("lba          : %#"PRIx64"\n",(uint64_t)le64_to_cpu(err_log[i].lba));
 		printf("nsid         : %#x\n", err_log[i].nsid);
 		printf("vs           : %d\n", err_log[i].vs);
 		printf(".................\n");
@@ -699,17 +717,17 @@ void show_nvme_resv_report(struct nvme_reservation_status *status)
 	regctl = status->regctl[0] | (status->regctl[1] << 8);
 
 	printf("\nNVME Reservation status:\n\n");
-	printf("gen       : %d\n", le32toh(status->gen));
+	printf("gen       : %d\n", le32_to_cpu(status->gen));
 	printf("regctl    : %d\n", regctl);
 	printf("rtype     : %d\n", status->rtype);
 	printf("ptpls     : %d\n", status->ptpls);
 
 	for (i = 0; i < regctl; i++) {
 		printf("regctl[%d] :\n", i);
-		printf("  cntlid  : %x\n", le16toh(status->regctl_ds[i].cntlid));
+		printf("  cntlid  : %x\n", le16_to_cpu(status->regctl_ds[i].cntlid));
 		printf("  rcsts   : %x\n", status->regctl_ds[i].rcsts);
-		printf("  hostid  : %"PRIx64"\n", (uint64_t)le64toh(status->regctl_ds[i].hostid));
-		printf("  rkey    : %"PRIx64"\n", (uint64_t)le64toh(status->regctl_ds[i].rkey));
+		printf("  hostid  : %"PRIx64"\n", (uint64_t)le64_to_cpu(status->regctl_ds[i].hostid));
+		printf("  rkey    : %"PRIx64"\n", (uint64_t)le64_to_cpu(status->regctl_ds[i].rkey));
 	}
 	printf("\n");
 }
@@ -784,10 +802,10 @@ void show_smart_log(struct nvme_smart_log *smart, unsigned int nsid, const char
 		int128_to_double(smart->media_errors));
 	printf("num_err_log_entries                 : %'.0Lf\n",
 		int128_to_double(smart->num_err_log_entries));
-	printf("Warning Temperature Time            : %u\n", le32toh(smart->warning_temp_time));
-	printf("Critical Composite Temperature Time : %u\n", le32toh(smart->critical_comp_time));
+	printf("Warning Temperature Time            : %u\n", le32_to_cpu(smart->warning_temp_time));
+	printf("Critical Composite Temperature Time : %u\n", le32_to_cpu(smart->critical_comp_time));
 	for (c=0; c < 8; c++) {
-		__u16 temp = le16toh(smart->temp_sensor[c]);
+		__u16 temp = le16_to_cpu(smart->temp_sensor[c]);
 		printf("Temperature Sensor %d                : %u C\n", c + 1,
 			temp ? temp - 273 : 0);
 	}
@@ -805,9 +823,9 @@ void show_intel_smart_log(struct nvme_additional_smart_log *smart, unsigned int
 		int48_to_long(smart->erase_fail_cnt.raw));
 	printf("wear_leveling                   : %3d%%       min: %u, max: %u, avg: %u\n",
 		smart->wear_leveling_cnt.norm,
-		le16toh(smart->wear_leveling_cnt.wear_level.min),
-		le16toh(smart->wear_leveling_cnt.wear_level.max),
-		le16toh(smart->wear_leveling_cnt.wear_level.avg));
+		le16_to_cpu(smart->wear_leveling_cnt.wear_level.min),
+		le16_to_cpu(smart->wear_leveling_cnt.wear_level.max),
+		le16_to_cpu(smart->wear_leveling_cnt.wear_level.avg));
 	printf("end_to_end_error_detection_count: %3d%%       %lu\n",
 		smart->e2e_err_cnt.norm,
 		int48_to_long(smart->e2e_err_cnt.raw));
@@ -1119,6 +1137,54 @@ void nvme_feature_show_fields(__u32 fid, unsigned int result, unsigned char *buf
 	}
 }
 
+void json_print_list_items(struct list_item *list_items, unsigned len)
+{
+	struct json_object *root;
+	struct json_object *device_attrs;
+	char formatter[41] = { 0 };
+	int index, i = 0;
+	char *product;
+
+	root = json_create_object();
+	for (i = 0; i < len; i++) {
+		device_attrs = json_create_object();
+
+		json_object_add_value_string(device_attrs,
+					     "DevicePath",
+					     list_items[i].node);
+		json_object_add_value_string(device_attrs,
+					     "Firmware",
+					     list_items[i].ctrl.fr);
+
+		if (sscanf(list_items[i].node, "/dev/nvme%d", &index) == 1)
+			json_object_add_value_int(device_attrs,
+						  "Index",
+						  index);
+
+		format(formatter, sizeof(formatter),
+		       list_items[i].ctrl.mn,
+		       sizeof(list_items[i].ctrl.mn));
+
+		json_object_add_value_string(device_attrs,
+					     "ModelNumber",
+					     formatter);
+
+		product = nvme_product_name(index);
+
+		format(formatter, sizeof(formatter),
+		       list_items[i].ctrl.sn,
+		       sizeof(list_items[i].ctrl.sn));
+
+		json_object_add_value_string(device_attrs,
+					     "SerialNumber",
+					     formatter);
+
+		json_object_add_value_object(root, product, device_attrs);
+		free((void*)product);
+	}
+	json_print_object(root, NULL);
+}
+
 void json_nvme_id_ns(struct nvme_id_ns *ns, unsigned int mode)
 {
 	char nguid_buf[2 * sizeof(ns->nguid) + 1], eui64_buf[2 * sizeof(ns->eui64) + 1];
@@ -1131,9 +1197,9 @@ void json_nvme_id_ns(struct nvme_id_ns *ns, unsigned int mode)
 
 	root = json_create_object();
 
-	json_object_add_value_int(root, "nsze", ns->nsze);
-	json_object_add_value_int(root, "ncap", ns->ncap);
-	json_object_add_value_int(root, "nuse", ns->nuse);
+	json_object_add_value_int(root, "nsze", le64_to_cpu(ns->nsze));
+	json_object_add_value_int(root, "ncap", le64_to_cpu(ns->ncap));
+	json_object_add_value_int(root, "nuse", le64_to_cpu(ns->nuse));
 	json_object_add_value_int(root, "nsfeat", ns->nsfeat);
 	json_object_add_value_int(root, "nlbaf", ns->nlbaf);
 	json_object_add_value_int(root, "flbas", ns->flbas);
@@ -1143,12 +1209,12 @@ void json_nvme_id_ns(struct nvme_id_ns *ns, unsigned int mode)
 	json_object_add_value_int(root, "nmic", ns->nmic);
 	json_object_add_value_int(root, "rescap", ns->rescap);
 	json_object_add_value_int(root, "fpi", ns->fpi);
-	json_object_add_value_int(root, "nawun", ns->nawun);
-	json_object_add_value_int(root, "nawupf", ns->nawupf);
-	json_object_add_value_int(root, "nacwu", ns->nacwu);
-	json_object_add_value_int(root, "nabsn", ns->nabsn);
-	json_object_add_value_int(root, "nabo", ns->nabo);
-	json_object_add_value_int(root, "nabspf", ns->nabspf);
+	json_object_add_value_int(root, "nawun", le16_to_cpu(ns->nawun));
+	json_object_add_value_int(root, "nawupf", le16_to_cpu(ns->nawupf));
+	json_object_add_value_int(root, "nacwu", le16_to_cpu(ns->nacwu));
+	json_object_add_value_int(root, "nabsn", le16_to_cpu(ns->nabsn));
+	json_object_add_value_int(root, "nabo", le16_to_cpu(ns->nabo));
+	json_object_add_value_int(root, "nabspf", le16_to_cpu(ns->nabspf));
 	json_object_add_value_float(root, "nvmcap", nvmcap);
 
 	memset(eui64, 0, sizeof(eui64_buf));
@@ -1168,9 +1234,9 @@ void json_nvme_id_ns(struct nvme_id_ns *ns, unsigned int mode)
 	for (i = 0; i <= ns->nlbaf; i++) {
 		struct json_object *lbaf = json_create_object();
 
-		json_object_add_value_int(lbaf, "ms", ns->lbaf[i].ms);
-		json_object_add_value_int(lbaf, "ds", ns->lbaf[i].ds);
-		json_object_add_value_int(lbaf, "rp", ns->lbaf[i].rp);
+		json_object_add_value_int(lbaf, "ms", le16_to_cpu(ns->lbaf[i].ms));
+		json_object_add_value_int(lbaf, "ds", le16_to_cpu(ns->lbaf[i].ds));
+		json_object_add_value_int(lbaf, "rp", le16_to_cpu(ns->lbaf[i].rp));
 
 		json_array_add_value_object(lbafs, lbaf);
 	}
@@ -1188,28 +1254,34 @@ void json_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode)
 	long double unvmcap = int128_to_double(ctrl->unvmcap);
 
 	char sn[sizeof(ctrl->sn) + 1], mn[sizeof(ctrl->mn) + 1], fr[sizeof(ctrl->fr) + 1];
+	char subnqn[sizeof(ctrl->subnqn) + 1];
 	__u32 ieee = ctrl->ieee[2] << 16 | ctrl->ieee[1] << 8 | ctrl->ieee[0];
 
 	int i;
 
-	sprintf(sn, "%-.*s\n", (int)sizeof(ctrl->sn), ctrl->sn);
-	sprintf(mn, "%-.*s\n", (int)sizeof(ctrl->mn), ctrl->mn);
-	sprintf(fr, "%-.*s\n", (int)sizeof(ctrl->fr), ctrl->fr);
+	snprintf(sn, sizeof(sn), "%-.*s\n", (int)sizeof(ctrl->sn), ctrl->sn);
+	snprintf(mn, sizeof(mn), "%-.*s\n", (int)sizeof(ctrl->mn), ctrl->mn);
+	snprintf(fr, sizeof(fr), "%-.*s\n", (int)sizeof(ctrl->fr), ctrl->fr);
+	snprintf(subnqn, sizeof(subnqn), "%-.*s\n", (int)sizeof(ctrl->subnqn), ctrl->subnqn);
+
 
 	root = json_create_object();
 
-	json_object_add_value_int(root, "vid", ctrl->vid);
-	json_object_add_value_int(root, "ssvid", ctrl->ssvid);
+	json_object_add_value_int(root, "vid", le16_to_cpu(ctrl->vid));
+	json_object_add_value_int(root, "ssvid", le16_to_cpu(ctrl->ssvid));
+	json_object_add_value_string(root, "sn", sn);
+	json_object_add_value_string(root, "mn", mn);
+	json_object_add_value_string(root, "fr", fr);
 	json_object_add_value_int(root, "rab", ctrl->rab);
 	json_object_add_value_int(root, "ieee", ieee);
 	json_object_add_value_int(root, "cmic", ctrl->cmic);
 	json_object_add_value_int(root, "mdts", ctrl->mdts);
-	json_object_add_value_int(root, "cntlid", ctrl->cntlid);
-	json_object_add_value_int(root, "ver", ctrl->ver);
-	json_object_add_value_int(root, "rtd3r", ctrl->rtd3r);
-	json_object_add_value_int(root, "rtd3e", ctrl->rtd3e);
-	json_object_add_value_int(root, "oaes", ctrl->oaes);
-	json_object_add_value_int(root, "oacs", ctrl->oacs);
+	json_object_add_value_int(root, "cntlid", le16_to_cpu(ctrl->cntlid));
+	json_object_add_value_int(root, "ver", le32_to_cpu(ctrl->ver));
+	json_object_add_value_int(root, "rtd3r", le32_to_cpu(ctrl->rtd3r));
+	json_object_add_value_int(root, "rtd3e", le32_to_cpu(ctrl->rtd3e));
+	json_object_add_value_int(root, "oaes", le32_to_cpu(ctrl->oaes));
+	json_object_add_value_int(root, "oacs", le16_to_cpu(ctrl->oacs));
 	json_object_add_value_int(root, "acl", ctrl->acl);
 	json_object_add_value_int(root, "aerl", ctrl->aerl);
 	json_object_add_value_int(root, "frmw", ctrl->frmw);
@@ -1218,26 +1290,27 @@ void json_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode)
 	json_object_add_value_int(root, "npss", ctrl->npss);
 	json_object_add_value_int(root, "avscc", ctrl->avscc);
 	json_object_add_value_int(root, "apsta", ctrl->apsta);
-	json_object_add_value_int(root, "wctemp", ctrl->wctemp);
-	json_object_add_value_int(root, "cctemp", ctrl->cctemp);
-	json_object_add_value_int(root, "mtfa", ctrl->mtfa);
-	json_object_add_value_int(root, "hmpre", ctrl->hmpre);
-	json_object_add_value_int(root, "hmmin", ctrl->hmmin);
+	json_object_add_value_int(root, "wctemp", le16_to_cpu(ctrl->wctemp));
+	json_object_add_value_int(root, "cctemp", le16_to_cpu(ctrl->cctemp));
+	json_object_add_value_int(root, "mtfa", le16_to_cpu(ctrl->mtfa));
+	json_object_add_value_int(root, "hmpre", le32_to_cpu(ctrl->hmpre));
+	json_object_add_value_int(root, "hmmin", le32_to_cpu(ctrl->hmmin));
 	json_object_add_value_float(root, "tnvmcap", tnvmcap);
 	json_object_add_value_float(root, "unvmcap", unvmcap);
-	json_object_add_value_int(root, "rpmbs", ctrl->rpmbs);
+	json_object_add_value_int(root, "rpmbs", le32_to_cpu(ctrl->rpmbs));
 	json_object_add_value_int(root, "sqes", ctrl->sqes);
 	json_object_add_value_int(root, "cqes", ctrl->cqes);
-	json_object_add_value_int(root, "nn", ctrl->nn);
-	json_object_add_value_int(root, "oncs", ctrl->oncs);
-	json_object_add_value_int(root, "fuses", ctrl->fuses);
+	json_object_add_value_int(root, "nn", le32_to_cpu(ctrl->nn));
+	json_object_add_value_int(root, "oncs", le16_to_cpu(ctrl->oncs));
+	json_object_add_value_int(root, "fuses", le16_to_cpu(ctrl->fuses));
 	json_object_add_value_int(root, "fna", ctrl->fna);
 	json_object_add_value_int(root, "vwc", ctrl->vwc);
-	json_object_add_value_int(root, "awun", ctrl->awun);
-	json_object_add_value_int(root, "awupf", ctrl->awupf);
+	json_object_add_value_int(root, "awun", le16_to_cpu(ctrl->awun));
+	json_object_add_value_int(root, "awupf", le16_to_cpu(ctrl->awupf));
 	json_object_add_value_int(root, "nvscc", ctrl->nvscc);
-	json_object_add_value_int(root, "acwu", ctrl->acwu);
-	json_object_add_value_int(root, "sgls", ctrl->sgls);
+	json_object_add_value_int(root, "acwu", le16_to_cpu(ctrl->acwu));
+	json_object_add_value_int(root, "sgls", le32_to_cpu(ctrl->sgls));
+	json_object_add_value_string(root, "subnqn", subnqn);
 
 	psds = json_create_array();
 	json_object_add_value_array(root, "psds", psds);
@@ -1245,17 +1318,22 @@ void json_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode)
 	for (i = 0; i <= ctrl->npss; i++) {
 		struct json_object *psd = json_create_object();
 
-		json_object_add_value_int(psd, "max_power", ctrl->psd[i].max_power);
+		json_object_add_value_int(psd, "max_power",
+			le16_to_cpu(ctrl->psd[i].max_power));
 		json_object_add_value_int(psd, "flags", ctrl->psd[i].flags);
-		json_object_add_value_int(psd, "entry_lat", ctrl->psd[i].entry_lat);
-		json_object_add_value_int(psd, "exit_lat", ctrl->psd[i].exit_lat);
+		json_object_add_value_int(psd, "entry_lat",
+			le32_to_cpu(ctrl->psd[i].entry_lat));
+		json_object_add_value_int(psd, "exit_lat",
+			le32_to_cpu(ctrl->psd[i].exit_lat));
 		json_object_add_value_int(psd, "read_tput", ctrl->psd[i].read_tput);
 		json_object_add_value_int(psd, "read_lat", ctrl->psd[i].read_lat);
 		json_object_add_value_int(psd, "write_tput", ctrl->psd[i].write_tput);
 		json_object_add_value_int(psd, "write_lat", ctrl->psd[i].write_lat);
-		json_object_add_value_int(psd, "idle_power", ctrl->psd[i].idle_power);
+		json_object_add_value_int(psd, "idle_power",
+			le16_to_cpu(ctrl->psd[i].idle_power));
 		json_object_add_value_int(psd, "idle_scale", ctrl->psd[i].idle_scale);
-		json_object_add_value_int(psd, "active_power", ctrl->psd[i].active_power);
+		json_object_add_value_int(psd, "active_power",
+			le16_to_cpu(ctrl->psd[i].active_power));
 		json_object_add_value_int(psd, "active_work_scale", ctrl->psd[i].active_work_scale);
 
 		json_array_add_value_object(psds, psd);
@@ -1306,7 +1384,7 @@ void json_nvme_resv_report(struct nvme_reservation_status *status)
 
 	root = json_create_object();
 
-	json_object_add_value_int(root, "gen", le32toh(status->gen));
+	json_object_add_value_int(root, "gen", le32_to_cpu(status->gen));
 	json_object_add_value_int(root, "regctl", regctl);
 	json_object_add_value_int(root, "rtype", status->rtype);
 	json_object_add_value_int(root, "ptpls", status->ptpls);
@@ -1317,10 +1395,10 @@ void json_nvme_resv_report(struct nvme_reservation_status *status)
 	for (i = 0; i < regctl; i++) {
 		struct json_object *rc = json_create_object();
 
-		json_object_add_value_int(rc, "cntlid", le16toh(status->regctl_ds[i].cntlid));
+		json_object_add_value_int(rc, "cntlid", le16_to_cpu(status->regctl_ds[i].cntlid));
 		json_object_add_value_int(rc, "rcsts", status->regctl_ds[i].rcsts);
-		json_object_add_value_int(rc, "hostid", (uint64_t)le64toh(status->regctl_ds[i].hostid));
-		json_object_add_value_int(rc, "rkey", (uint64_t)le64toh(status->regctl_ds[i].rkey));
+		json_object_add_value_int(rc, "hostid", (uint64_t)le64_to_cpu(status->regctl_ds[i].hostid));
+		json_object_add_value_int(rc, "rkey", (uint64_t)le64_to_cpu(status->regctl_ds[i].rkey));
 
 		json_array_add_value_object(rcs, rc);
 	}
@@ -1332,29 +1410,79 @@ void json_nvme_resv_report(struct nvme_reservation_status *status)
 void json_fw_log(struct nvme_firmware_log_page *fw_log, const char *devname)
 {
 	struct json_object *root;
-	struct json_array *frs;
+	struct json_object *fwsi;
+	char fmt[21];
+	char str[32];
 	int i;
 
 	root = json_create_object();
+	fwsi = json_create_object();
 
-	json_object_add_value_int(root, "afi", fw_log->afi);
-
-	frs = json_create_array();
-	json_object_add_value_array(root, "frs", frs);
+	json_object_add_value_int(fwsi, "Active Firmware Slot (afi)", fw_log->afi);
 
 	for (i = 0; i < 7; i++) {
-		struct json_object *fr = json_create_object();
-
-		json_object_add_value_int(fr, "frs", fw_log->frs[i]);
-
-		json_array_add_value_object(frs, fr);
-
+		snprintf(fmt, sizeof(fmt), "Firmware Rev Slot %d", i);
+		snprintf(str, sizeof(str), "%lu (%s)", (uint64_t)fw_log->frs[i],
+			 fw_to_string(fw_log->frs[i]));
+		json_object_add_value_string(fwsi, fmt, str);
 	}
+	json_object_add_value_object(root, devname, fwsi);
 
 	json_print_object(root, NULL);
 	printf("\n");
 }
 
+void json_add_smart_log(struct nvme_additional_smart_log *smart,
+			unsigned int nsid, const char *devname)
+{
+	struct json_object *root;
+	struct json_object *data;
+	char fmt[128];
+
+	root = json_create_object();
+	data = json_create_object();
+
+	json_object_add_value_int(data, "Program Fail Count",
+				  int48_to_long(smart->program_fail_cnt.raw));
+	json_object_add_value_int(data, "Erase Fail Count",
+				  int48_to_long(smart->erase_fail_cnt.raw));
+	json_object_add_value_int(data, "Wear Leveling Min",
+				  le16toh(smart->wear_leveling_cnt.wear_level.min));
+	json_object_add_value_int(data, "Wear Leveling Max",
+				  le16toh(smart->wear_leveling_cnt.wear_level.max));
+	json_object_add_value_int(data, "Wear Leveling Avg",
+				  le16toh(smart->wear_leveling_cnt.wear_level.avg));
+	json_object_add_value_int(data, "End-to-end Error Detection Count",
+				  int48_to_long(smart->e2e_err_cnt.raw));
+	json_object_add_value_int(data, "CRC Error Count",
+				  int48_to_long(smart->crc_err_cnt.raw));
+	json_object_add_value_float(data, "Timed Workload Media Wear",
+				    ((long double)int48_to_long(smart->timed_workload_media_wear.raw)) / 1024);
+
+	json_object_add_value_int(data, "Timed Workload Host Reads",
+				  int48_to_long(smart->timed_workload_host_reads.raw));
+	json_object_add_value_int(data, "Timed Workload Timer",
+				  int48_to_long(smart->timed_workload_timer.raw));
+	snprintf(fmt, sizeof(fmt), "%u%%",
+		 smart->thermal_throttle_status.thermal_throttle.pct);
+	json_object_add_value_string(data, "Thermal Throttle status Percentage",
+				     fmt);
+	json_object_add_value_int(data, "Thermal Throttle Status Count",
+				  smart->thermal_throttle_status.thermal_throttle.count);
+	json_object_add_value_int(data, "Retry Buffer Overflow Count",
+				  int48_to_long(smart->retry_buffer_overflow_cnt.raw));
+	json_object_add_value_int(data, "PLL Lock Loss Count",
+				  int48_to_long(smart->pll_lock_loss_cnt.raw));
+	json_object_add_value_int(data, "Nand Bytes Written",
+				  int48_to_long(smart->nand_bytes_written.raw));
+	json_object_add_value_int(data, "Host Bytes Written",
+				  int48_to_long(smart->host_bytes_written.raw));
+
+	snprintf(fmt, sizeof(fmt), "Additional Smart Log for %s", devname);
+	json_object_add_value_object(root, fmt, data);
+	json_print_object(root, NULL);
+}
+
 void json_smart_log(struct nvme_smart_log *smart, unsigned int nsid, const char *devname)
 {
 	struct json_object *root;
@@ -1390,8 +1518,10 @@ void json_smart_log(struct nvme_smart_log *smart, unsigned int nsid, const char
 	json_object_add_value_float(root, "unsafe_shutdowns", unsafe_shutdowns);
 	json_object_add_value_float(root, "media_errors", media_errors);
 	json_object_add_value_float(root, "num_err_log_entries", num_err_log_entries);
-	json_object_add_value_int(root, "warning_temp_time", smart->warning_temp_time);
-	json_object_add_value_int(root, "critical_comp_time", smart->critical_comp_time);
+	json_object_add_value_int(root, "warning_temp_time",
+			le32_to_cpu(smart->warning_temp_time));
+	json_object_add_value_int(root, "critical_comp_time",
+			le32_to_cpu(smart->critical_comp_time));
 
 	json_print_object(root, NULL);
 	printf("\n");
@@ -1417,7 +1547,7 @@ void show_registers_version(__u32 vs)
 	printf("\tNVMe specification %d.%d\n\n", (vs & 0xffff0000) >> 16,  (vs & 0x0000ff00) >> 8);
 }
 
-void show_registers_cc_ams (__u8 ams)
+static void show_registers_cc_ams (__u8 ams)
 {
 	printf("\tArbitration Mechanism Selected     (AMS): ");
 	switch (ams) {
@@ -1435,7 +1565,7 @@ void show_registers_cc_ams (__u8 ams)
 	}
 }
 
-void show_registers_cc_shn (__u8 shn)
+static void show_registers_cc_shn (__u8 shn)
 {
 	printf("\tShutdown Notification              (SHN): ");
 	switch (shn) {
@@ -1464,7 +1594,7 @@ void show_registers_cc (__u32 cc)
 	printf("\tEnable                              (EN): %s\n\n", (cc & 0x00000001) ? "Yes":"No");
 }
 
-void show_registers_csts_shst (__u8 shst)
+static void show_registers_csts_shst (__u8 shst)
 {
 	printf("\tShutdown Status               (SHST): ");
 	switch (shst) {
@@ -1510,7 +1640,7 @@ void show_registers_cmbloc(__u32 cmbloc, __u32 cmbsz)
 	}
 }
 
-char *nvme_register_szu_to_string(__u8 szu)
+static char *nvme_register_szu_to_string(__u8 szu)
 {
 	switch (szu) {
 	case 0:	return "4 KB";
